/*
 *  Copyright (c) 2004 by Luis G. Lopez <llopez@cnba.uba.ar>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/**
* This is the main class.<p>
*
* It provides useful methods to calculate the mean value of a serie of numbers, and its standard deviation and more.
*
* @author    <A HREF="mailto:llopez@cnba.uba.ar">Luis G. Lpez</A>, Argentina
* @version   0.1 (12 Abril 2004)
**/
public class estadistica {

	private double promedio;
	private double sigmaN;
	private double sigmaN_1;
	private double errAbs;
	private int N;
	private int minimo1;
	private int minimo2;

	/** This constructor sets <b>minimo1</b> and <b>minimo2</b> to their defalut values (<b>10</b> and <b>30</b>) <p>
	*
	*	The meaning of these values are explain latter, in the <b>getErrAbs</b> method.
	*
	*/
	public estadistica() {
		minimo1 = 10;
		minimo2 = 30;
	}

	/**
	*	Sets the number of measures to be taken into account.
	*
	*	@param n	number of measures
	*/
	public void setN(int n) {
		N = n;
	}

	/**
	*	Sets the value of <b>minimum1</b>.
	*
	*	@param m	the value of minimum1
	*/
	public void setMinimo1 (int m) {
		minimo1 = m;
	}

	/**
	*	Sets the value of <b>minimum2</b>.
	*
	*	@param m	the value of minimum2
	*/
	public void setMinimo2 (int m) {
		minimo2 = m;
	}

	/**
	*	Gets the mean value a serie of values.
	*
	*	@param data[]	array of values
	*/
	public double getPromedio (double [] data) {
		promedio = 0;
		final int n = data.length; 
		if (n == 1) return data[0]; 
		for (int i=0; i < data.length; i++) { 
			promedio += data[i];
		}	
		promedio = promedio/n;
		return promedio;
	}

	/**
	*	Calculates the <b>sigma<sub>N</sub></b> of a serie of values.
	*
	*	@param data[]	array of values
	*/
	public double getSigmaN (double [] data) {
		sigmaN = sdKnuth (data, 0);
		return sigmaN;
	}

	/**
	*	Calculates the <b>sigma<sub>N-1</sub></b> of a serie of values.
	*
	*	@param data[]	array of values
	*/
	public double getSigmaN_1 (double [] data) {
		sigmaN_1 = sdKnuth (data, 1);
		return sigmaN_1;
	}

	/**
	*	Calculates the <b>absolute error</b> of a serie of values. <p>
	*
	*	If the number of values is equal or less than <b>minimum1</b>, it simply calculates the mean value 
	*	between the maximum and minimun value of the array <p>
	*	
	*	If the number of values is greater than <b>minimum1</b> and equal or less than <b>minimum2</b>, it divides 
	*	<b>sigma<sub>N-1</sub></b> and the square root of the number of values, and multiplies the result by 2.
	*	
	*	If the number of values is greater than <b>minimum2</b>, it divides 
	*	<b>sigma<sub>N</sub></b> and the square root of the number of values, and multiplies the result by 2.
	*
	*	@param data[]	array of values
	*/
	public double getErrAbs (double [] data) {
		N = data.length; 
		if (N == 1) {
			errAbs = 0;
		} else if (N <= minimo1) {
			double minimo = data[0];
			double maximo = data[0];
			for (int i=1; i<N; i++)	{
				minimo = Math.min(minimo, data[i]);
				maximo = Math.max(maximo, data[i]);
				errAbs = (maximo - minimo) / 2;
			}
		} else if (N > minimo1 && N <= minimo2) {
			errAbs = getSigmaN_1(data) / Math.sqrt(N) * 2.0;
		} else {
			errAbs = getSigmaN(data) / Math.sqrt(N) * 2.0;
		}
		return errAbs;
	}

	/** 
	* Calculates the standard deviation of an array 
	* of numbers. 
	* see Knuth's The Art Of Computer Programming 
	* Volume II: Seminumerical Algorithms 
	* This algorithm is slower, but more resistant to error propagation. 
	* 
	* @param data Numbers to compute the standard deviation of. 
	* Array must contain two or more numbers. 
	* @return standard deviation estimate of population 
	* ( to get estimate of sample, use n instead of n-1 in last line ) 
	*/ 
	public double sdKnuth (double[] data, int minus) { 
		final int n = data.length; 
		if (n == 1) return 0; 
		double avg = data[0]; 
		double sum = 0; 
		for (int i=1; i < data.length; i++) { 
			double newavg = avg + ( data[ i ] - avg) / (i + 1) ; 
			sum += ( data[ i ] - avg) * (data [ i ] - newavg); 
			avg = newavg ; 
		} 
		return Math.sqrt (sum / ( n - minus )); 
	} 
}